@@ -13,11 +13,10 @@ from acme import messages
 
 from certbot import account
 from certbot import errors
-from certbot import interfaces
 
 from certbot.display import util as display_util
 
-from certbot.tests import test_util
+import certbot.tests.util as test_util
 
 
 KEY = jose.JWKRSA.load(test_util.load_vector("rsa512_key.pem"))
@@ -26,57 +25,73 @@ KEY = jose.JWKRSA.load(test_util.load_vector("rsa512_key.pem"))
 class GetEmailTest(unittest.TestCase):
     """Tests for certbot.display.ops.get_email."""
 
-    def setUp(self):
-        mock_display = mock.MagicMock()
-        self.input = mock_display.input
-        zope.component.provideUtility(mock_display, interfaces.IDisplay)
-
     @classmethod
     def _call(cls, **kwargs):
         from certbot.display.ops import get_email
         return get_email(**kwargs)
 
-    def test_cancel_none(self):
-        self.input.return_value = (display_util.CANCEL, "foo@bar.baz")
+    @test_util.patch_get_utility("certbot.display.ops.z_util")
+    def test_cancel_none(self, mock_get_utility):
+        mock_input = mock_get_utility().input
+        mock_input.return_value = (display_util.CANCEL, "foo@bar.baz")
         self.assertRaises(errors.Error, self._call)
         self.assertRaises(errors.Error, self._call, optional=False)
 
-    def test_ok_safe(self):
-        self.input.return_value = (display_util.OK, "foo@bar.baz")
+    @test_util.patch_get_utility("certbot.display.ops.z_util")
+    def test_ok_safe(self, mock_get_utility):
+        mock_input = mock_get_utility().input
+        mock_input.return_value = (display_util.OK, "foo@bar.baz")
         with mock.patch("certbot.display.ops.util.safe_email") as mock_safe_email:
             mock_safe_email.return_value = True
             self.assertTrue(self._call() is "foo@bar.baz")
 
-    def test_ok_not_safe(self):
-        self.input.return_value = (display_util.OK, "foo@bar.baz")
+    @test_util.patch_get_utility("certbot.display.ops.z_util")
+    def test_ok_not_safe(self, mock_get_utility):
+        mock_input = mock_get_utility().input
+        mock_input.return_value = (display_util.OK, "foo@bar.baz")
         with mock.patch("certbot.display.ops.util.safe_email") as mock_safe_email:
             mock_safe_email.side_effect = [False, True]
             self.assertTrue(self._call() is "foo@bar.baz")
 
-    def test_invalid_flag(self):
+    @test_util.patch_get_utility("certbot.display.ops.z_util")
+    def test_invalid_flag(self, mock_get_utility):
         invalid_txt = "There seem to be problems"
-        self.input.return_value = (display_util.OK, "foo@bar.baz")
+        mock_input = mock_get_utility().input
+        mock_input.return_value = (display_util.OK, "foo@bar.baz")
         with mock.patch("certbot.display.ops.util.safe_email") as mock_safe_email:
             mock_safe_email.return_value = True
             self._call()
-            self.assertTrue(invalid_txt not in self.input.call_args[0][0])
+            self.assertTrue(invalid_txt not in mock_input.call_args[0][0])
             self._call(invalid=True)
-            self.assertTrue(invalid_txt in self.input.call_args[0][0])
+            self.assertTrue(invalid_txt in mock_input.call_args[0][0])
 
-    def test_optional_flag(self):
-        self.input.return_value = (display_util.OK, "foo@bar.baz")
+    @test_util.patch_get_utility("certbot.display.ops.z_util")
+    def test_optional_flag(self, mock_get_utility):
+        mock_input = mock_get_utility().input
+        mock_input.return_value = (display_util.OK, "foo@bar.baz")
         with mock.patch("certbot.display.ops.util.safe_email") as mock_safe_email:
             mock_safe_email.side_effect = [False, True]
             self._call(optional=False)
-            for call in self.input.call_args_list:
+            for call in mock_input.call_args_list:
                 self.assertTrue(
                     "--register-unsafely-without-email" not in call[0][0])
 
+    @test_util.patch_get_utility("certbot.display.ops.z_util")
+    def test_optional_invalid_unsafe(self, mock_get_utility):
+        invalid_txt = "There seem to be problems"
+        mock_input = mock_get_utility().input
+        mock_input.return_value = (display_util.OK, "foo@bar.baz")
+        with mock.patch("certbot.display.ops.util.safe_email") as mock_safe_email:
+            mock_safe_email.side_effect = [False, True]
+            self._call(invalid=True)
+            self.assertTrue(invalid_txt in mock_input.call_args[0][0])
+
 
 class ChooseAccountTest(unittest.TestCase):
     """Tests for certbot.display.ops.choose_account."""
     def setUp(self):
-        zope.component.provideUtility(display_util.FileDisplay(sys.stdout))
+        zope.component.provideUtility(display_util.FileDisplay(sys.stdout,
+                                                               False))
 
         self.accounts_dir = tempfile.mkdtemp("accounts")
         self.account_keys_dir = os.path.join(self.accounts_dir, "keys")
@@ -119,7 +134,8 @@ class ChooseAccountTest(unittest.TestCase):
 class GenSSLLabURLs(unittest.TestCase):
     """Loose test of _gen_ssl_lab_urls. URL can change easily in the future."""
     def setUp(self):
-        zope.component.provideUtility(display_util.FileDisplay(sys.stdout))
+        zope.component.provideUtility(display_util.FileDisplay(sys.stdout,
+                                                               False))
 
     @classmethod
     def _call(cls, domains):
@@ -138,7 +154,8 @@ class GenSSLLabURLs(unittest.TestCase):
 class GenHttpsNamesTest(unittest.TestCase):
     """Test _gen_https_names."""
     def setUp(self):
-        zope.component.provideUtility(display_util.FileDisplay(sys.stdout))
+        zope.component.provideUtility(display_util.FileDisplay(sys.stdout,
+                                                               False))
 
     @classmethod
     def _call(cls, domains):
@@ -185,7 +202,8 @@ class GenHttpsNamesTest(unittest.TestCase):
 class ChooseNamesTest(unittest.TestCase):
     """Test choose names."""
     def setUp(self):
-        zope.component.provideUtility(display_util.FileDisplay(sys.stdout))
+        zope.component.provideUtility(display_util.FileDisplay(sys.stdout,
+                                                               False))
         self.mock_install = mock.MagicMock()
 
     @classmethod
@@ -215,6 +233,45 @@ class ChooseNamesTest(unittest.TestCase):
         self.assertTrue(
             "configuration files" in mock_util().input.call_args[0][0])
 
+    def test_sort_names_trivial(self):
+        from certbot.display.ops import _sort_names
+
+        #sort an empty list
+        self.assertEqual(_sort_names([]), [])
+
+        #sort simple domains
+        some_domains = ["ex.com", "zx.com", "ax.com"]
+        self.assertEqual(_sort_names(some_domains), ["ax.com", "ex.com", "zx.com"])
+
+        #Sort subdomains of a single domain
+        domain = ".ex.com"
+        unsorted_short = ["e", "a", "z", "y"]
+        unsorted_long = [us + domain for us in unsorted_short]
+
+        sorted_short = sorted(unsorted_short)
+        sorted_long = [us + domain for us in sorted_short]
+
+        self.assertEqual(_sort_names(unsorted_long), sorted_long)
+
+    def test_sort_names_many(self):
+        from certbot.display.ops import _sort_names
+
+        unsorted_domains = [".cx.com", ".bx.com", ".ax.com", ".dx.com"]
+        unsorted_short = ["www", "bnother.long.subdomain", "a", "a.long.subdomain", "z", "b"]
+        #Of course sorted doesn't work here ;-)
+        sorted_short = ["a", "b", "a.long.subdomain", "bnother.long.subdomain", "www", "z"]
+
+        to_sort = []
+        for short in unsorted_short:
+            for domain in unsorted_domains:
+                to_sort.append(short+domain)
+        sortd = []
+        for domain in sorted(unsorted_domains):
+            for short in sorted_short:
+                sortd.append(short+domain)
+        self.assertEqual(_sort_names(to_sort), sortd)
+
+
     @mock.patch("certbot.display.ops.z_util")
     def test_filter_names_valid_return(self, mock_util):
         self.mock_install.get_all_names.return_value = set(["example.com"])
@@ -244,8 +301,8 @@ class ChooseNamesTest(unittest.TestCase):
         all_valid = ["example.com", "second.example.com",
                      "also.example.com", "under_score.example.com",
                      "justtld"]
-        all_invalid = ["xn--ls8h.tld", "*.wildcard.com", "uniçodé.com"]
-        two_valid = ["example.com", "xn--ls8h.tld", "also.example.com"]
+        all_invalid = ["öóòps.net", "*.wildcard.com", "uniçodé.com"]
+        two_valid = ["example.com", "úniçøde.com", "also.example.com"]
         self.assertEqual(get_valid_domains(all_valid), all_valid)
         self.assertEqual(get_valid_domains(all_invalid), [])
         self.assertEqual(len(get_valid_domains(two_valid)), 2)
@@ -266,10 +323,6 @@ class ChooseNamesTest(unittest.TestCase):
             unicode_error = UnicodeEncodeError('mock', u'', 0, 1, 'mock')
             mock_sli.side_effect = unicode_error
             self.assertEqual(_choose_names_manually(), [])
-        # Punycode and no retry
-        mock_util().input.return_value = (display_util.OK,
-                                          "xn--ls8h.tld")
-        self.assertEqual(_choose_names_manually(), [])
         # Valid domains
         mock_util().input.return_value = (display_util.OK,
                                           ("example.com,"
@@ -317,7 +370,7 @@ class SuccessRenewalTest(unittest.TestCase):
     @classmethod
     def _call(cls, names):
         from certbot.display.ops import success_renewal
-        success_renewal(names, "renew")
+        success_renewal(names)
 
     @mock.patch("certbot.display.ops.z_util")
     def test_success_renewal(self, mock_util):
@@ -332,6 +385,21 @@ class SuccessRenewalTest(unittest.TestCase):
         for name in names:
             self.assertTrue(name in arg)
 
+class SuccessRevocationTest(unittest.TestCase):
+    # pylint: disable=too-few-public-methods
+    """Test the success revocation message."""
+    @classmethod
+    def _call(cls, path):
+        from certbot.display.ops import success_revocation
+        success_revocation(path)
+
+    @mock.patch("certbot.display.ops.z_util")
+    def test_success_revocation(self, mock_util):
+        mock_util().notification.return_value = None
+        path = "/path/to/cert.pem"
+        self._call(path)
+        mock_util().notification.assert_called_once()
+        self.assertTrue(path in mock_util().notification.call_args[0][0])
 
 if __name__ == "__main__":
     unittest.main()  # pragma: no cover
